/*
 * Copyright (c) 2020 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.misc;

import java.util.Objects;

import javax.annotation.Nonnull;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;

/**
 * Predicate that compares values of specified NBT keys subset.
 */
public class StackNbtMatcher implements StackMatchingPredicate {
    private String[] keys;

    public StackNbtMatcher(@Nonnull String ...keys) {
        this.keys = keys;
    }

    @Override
    public boolean isMatching(@Nonnull ItemStack base, @Nonnull ItemStack comparison) {
        NBTTagCompound baseNBT = base.func_77978_p();
        NBTTagCompound comparisonNBT = comparison.func_77978_p();

        for (String key : keys) {
            NBTBase baseValue = baseNBT != null ? baseNBT.func_74781_a(key) : null;
            NBTBase comparisonValue = comparisonNBT != null ? comparisonNBT.func_74781_a(key) : null;
            if (!Objects.equals(baseValue, comparisonValue)) {
                return false;
            }
        }

        return true;
    }
}
